@@ -20,29 +20,18 @@ package handlers
 import (
 	"encoding/json"
 	"errors"
-	"fmt"
 	"time"
 
 	"git.zabbix.com/ap/plugin-support/zbxerr"
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
+	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/mongo/options"
 )
 
-type oplogStats struct {
-	TimeDiff int `json:"timediff"` // in seconds
-}
-
-const (
-	oplogReplicaSet  = "oplog.rs"    // the capped collection that holds the oplog for Replica Set Members
-	oplogMasterSlave = "oplog.$main" // oplog for the master-slave configuration
-)
-
-var oplogQuery = bson.M{"ts": bson.M{"$exists": true}}
-
 // OplogStatsHandler
 // https://docs.mongodb.com/manual/reference/method/db.getReplicationInfo/index.html
-func OplogStatsHandler(s Session, _ map[string]string) (interface{}, error) {
+func OplogStatsHandler(s Session, _ map[string]string) (any, error) {
 	var (
 		err             error
 		firstTs, lastTs int
@@ -52,20 +41,29 @@ func OplogStatsHandler(s Session, _ map[string]string) (interface{}, error) {
 	findOptions := options.FindOne()
 	findOptions.SetMaxTime(time.Duration(s.GetMaxTimeMS()) * time.Millisecond)
 
-	for _, collection := range []string{oplogReplicaSet, oplogMasterSlave} {
+	for _, collection := range []string{
+		"oplog.rs",    // the capped collection that holds the oplog for Replica Set Members
+		"oplog.$main", // oplog for the master-slave configuration
+	} {
 		firstTs, lastTs, err = getTS(collection, localDb, findOptions)
 		if err != nil {
-			if errors.As(err, &errNotFound) {
-				continue
+			if !errors.Is(err, mongo.ErrNoDocuments) {
+				return nil, zbxerr.ErrorCannotFetchData.Wrap(err)
 			}
 
-			return nil, err
+			continue
 		}
 
 		break
 	}
 
-	jsonRes, err := json.Marshal(oplogStats{firstTs - lastTs})
+	jsonRes, err := json.Marshal(
+		struct {
+			TimeDiff int `json:"timediff"` // in seconds
+		}{
+			TimeDiff: firstTs - lastTs,
+		},
+	)
 	if err != nil {
 		return nil, zbxerr.ErrorCannotMarshalJSON.Wrap(err)
 	}
@@ -73,14 +71,20 @@ func OplogStatsHandler(s Session, _ map[string]string) (interface{}, error) {
 	return string(jsonRes), nil
 }
 
-func getTS(collection string, localDb Database, findOptions *options.FindOneOptions) (int, int, error) {
+func getTS(
+	collection string,
+	localDb Database,
+	findOptions *options.FindOneOptions,
+) (int, int, error) {
 	findOptions.SetSort(bson.D{{Key: sortNatural, Value: -1}})
+
 	firstTs, err := getOplogStats(localDb, collection, findOptions)
 	if err != nil {
 		return 0, 0, err
 	}
 
 	findOptions.SetSort(bson.D{{Key: sortNatural, Value: 1}})
+
 	lastTs, err := getOplogStats(localDb, collection, findOptions)
 	if err != nil {
 		return 0, 0, err
@@ -89,18 +93,21 @@ func getTS(collection string, localDb Database, findOptions *options.FindOneOpti
 	return firstTs, lastTs, nil
 }
 
-func getOplogStats(db Database, collection string, opt *options.FindOneOptions) (out int, err error) {
+func getOplogStats(
+	db Database,
+	collection string,
+	opt *options.FindOneOptions,
+) (int, error) {
 	var result primitive.D
 
-	err = db.C(collection).FindOne(oplogQuery, opt).GetSingle(&result)
+	err := db.C(collection).FindOne(bson.M{"ts": bson.M{"$exists": true}}, opt).
+		GetSingle(&result)
 	if err != nil {
-		if errors.As(err, &errNotFound) {
-			return
-		}
-
-		return out, zbxerr.ErrorCannotFetchData.Wrap(fmt.Errorf("failed to decode result: %w", err))
+		return 0, err
 	}
 
+	var out int
+
 	for _, op := range result {
 		if op.Key == timestampBSONName {
 			if pt, ok := op.Value.(primitive.Timestamp); ok {
@@ -109,5 +116,5 @@ func getOplogStats(db Database, collection string, opt *options.FindOneOptions)
 		}
 	}
 
-	return
+	return out, nil
 }
